<?php
/**
*
* @package phpBB3
* @version $Id: posts_anywhere.php 1202 2008-04-22 5:32:10Z handyman $
* @copyright (c) 2008 StarTrekGuide
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

define('LAST_POST', 0);
define('FIRST_POST', 1);

class posts_anywhere
{
	function display_instance($id)
	{
		global $db, $auth, $template, $user, $phpbb_root_path, $phpEx;

		$hilight = request_var('highlight', '');
		$forum_ary = $recursive_ary = $post_ary = $row_ary = $subforum_ary = array();

		// find out which forums and options we're asking for before we begin
		$sql_ary = array(
			'SELECT'	=> 'p.*, po.*',
			'FROM'		=> array(
				POSTS_ANYWHERE_TABLE 			=> 'p',
				POSTS_ANYWHERE_OPTIONS_TABLE	=> 'po',
			),
			'WHERE'		=> 'p.instance_id = ' . $id . '
				AND po.instance_id = p.instance_id',
		);
		$sql = $db->sql_build_query('SELECT', $sql_ary);
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$row_ary[$row['forum_id']] = $row;
			// if we want a single post, that's all we're gonna get
			if ($row['post_id'])
			{
				$post_ary[$row['post_id']] = $row['post_id'];
				continue;
			}

			// collect the forums we want to view recursively
			if ($row['recursive'])
			{
				$recursive_ary[$row['forum_id']] = $row['forum_id'];
			}

			// collect all the forum_id's for later use
			$forum_ary[$row['forum_id']] = $row['forum_id'];
		}
		$db->sql_freeresult($result);

		// Grab the forums recursively
		if (sizeof($recursive_ary))
		{
			$sql = $db->sql_build_query('SELECT', array(
				'SELECT'	=> 'a.forum_id as forum, b.forum_id',
				'FROM'		=> array(
					FORUMS_TABLE => 'a',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(FORUMS_TABLE => 'b'),
						'ON'	=> 'b.left_id BETWEEN a.left_id AND a.right_id',
					),
				),
				'WHERE'		=> $db->sql_in_set('a.forum_id', $recursive_ary),
			));
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				// yay, more forums to store in the forum_id array
				$subforum_ary[$row['forum']][$row['forum_id']] = $row['forum_id'];
			}
			$db->sql_freeresult($result);
		}

		// by this point, we should have all our forums together,

		// put the subforums in with the forums with the proper options
		foreach ($subforum_ary as $key => $var)
		{
			foreach ($var as $row)
			{
				// only add the options from the parent forum if we don't have this one already selected
				if (!isset($row_ary[$row]))
				{
					$row_ary[$row] = $row_ary[$key];
				}
			}
		}

		// now time to see which ones we have permission to view and weed out the others.
		foreach ($row_ary as $key => $var)
		{
			// remove the forums we don't have permission to view
			if ($auth->acl_get('!f_read', $key))
			{
				unset($row_ary[$key]);
			}
		}

		if (!sizeof($row_ary))
		{
			return;
		}

		// now we have all our forums
		// create an array of only forum_ids
		$first_post_ids = $last_post_ids = array();
		$last_post_limit = $first_post_limit = 0;
		foreach ($row_ary as $key => $row)
		{
			if ($row['post_limit'] == 1 && $row['single_post_topic'] == FIRST_POST)
			{
				$first_post_limit++;
				$first_post_ids[$key] = $key;
			}
			else
			{
				$last_post_limit++;
				$last_post_ids[$key] = $key;
			}
		}

		$post_ary = array();
		if (sizeof($first_post_ids))
		{
			$sql = 'SELECT topic_first_post_id
				FROM ' . TOPICS_TABLE . '
				WHERE ' . $db->sql_in_set('forum_id', $first_post_ids) . '
				ORDER BY post_time DESC';
			$result = $db->sql_query_limit($sql, $first_post_limit);

			while ($row = $db->sql_fetchrow($result))
			{
				$post_ary[] = $row['topic_first_post_id'];
			}
			$db->sql_freeresult($result);
		}

		if (sizeof($last_post_ids))
		{
			$sql = 'SELECT *
				FROM ' . POSTS_TABLE . '
				WHERE ' . $db->sql_in_set('forum_id', $last_post_ids) .
					(sizeof($post_ary) ? ' OR ' . $db->sql_in_set('post_id', $post_ary) : '') . '
				ORDER BY post_time DESC';
			$result = $db->sql_query_limit($sql, $first_post_limit + $last_post_limit);
			$postrow = $db->sql_fetchrowset($result);
			$db->sql_freeresult($result);

			// put together some stuff
			$poster_ids = $poster_ary = array();
			foreach ($postrow as $row)
			{
				$poster_ids[$row['poster_id']] = $row['poster_id'];
			}

			// get the users involved
			if (sizeof($poster_ids))
			{
				$sql = 'SELECT username, user_id, user_colour
					FROM ' . USERS_TABLE . '
					WHERE ' . $db->sql_in_set('user_id', $poster_ids);
				$result = $db->sql_query($sql);

				while ($row = $db->sql_fetchrow($result))
				{
					$poster_ary[$row['user_id']] = $row;
				}
				$db->sql_freeresult($result);
			}

			// add the users info to the postrow
			foreach ($postrow as $key => $row)
			{
				$row['username']	= $poster_ary[$row['poster_id']]['username'];
				$row['user_id']		= $poster_ary[$row['poster_id']]['user_id'];
				$row['user_colour']	= $poster_ary[$row['poster_id']]['user_colour'];

				// We pre-process some variables here for later usage
				$row['post_text'] = censor_text($row['post_text']);
				$text_only_message = $row['post_text'];
				// make list items visible as such
				if ($row['bbcode_uid'])
				{
					$text_only_message = str_replace('[*:' . $row['bbcode_uid'] . ']', '&sdot;&nbsp;', $text_only_message);
					// no BBCode in text only message
					strip_bbcode($text_only_message, $row['bbcode_uid']);
				}

				if (!$row_ary[$row['forum_id']]['character_limit'] || utf8_strlen($text_only_message) < ($row_ary[$row['forum_id']]['character_limit'] + 3))
				{
					$message = generate_text_for_display($row['post_text'], $row['bbcode_uid'], $row['bbcode_bitfield'], (($row['enable_bbcode']) ? OPTION_FLAG_BBCODE : 0) + (($row['enable_smilies']) ? OPTION_FLAG_SMILIES : 0) + (($row['enable_magic_url']) ? OPTION_FLAG_LINKS : 0));
				}
				else
				{
					// now find context for the searched words
					$message = get_context($text_only_message, array_filter(explode('|', $hilight), 'strlen'), $row_ary[$row['forum_id']]['character_limit']);
					$message = bbcode_nl2br($message);
				}

				$template->assign_block_vars('postrow', array(
					'POST_AUTHOR_FULL'		=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour'], $row['post_username']),
					'POST_AUTHOR_COLOUR'	=> get_username_string('colour', $row['user_id'], $row['username'], $row['user_colour'], $row['post_username']),
					'POST_AUTHOR'			=> get_username_string('username', $row['user_id'], $row['username'], $row['user_colour'], $row['post_username']),
					'U_POST_AUTHOR'			=> get_username_string('profile', $row['user_id'], $row['username'], $row['user_colour'], $row['post_username']),

					'POST_DATE'			=> $user->format_date($row['post_time']),
					'POST_SUBJECT'		=> $row['post_subject'],
					'MESSAGE'			=> $message,
					'EDIT_REASON'		=> $row['post_edit_reason'],

					'MINI_POST_IMG'			=> $user->img('icon_post_target', 'POST'),//($post_unread) ? $user->img('icon_post_target_unread', 'NEW_POST') : $user->img('icon_post_target', 'POST'),

					'U_EDIT'			=> (!$user->data['is_registered']) ? '' : ((($user->data['user_id'] == $row['user_id'] && $auth->acl_get('f_edit', $row['forum_id']) && ($row['post_time'] > time() - ($config['edit_time'] * 60) || !$config['edit_time'])) || $auth->acl_get('m_edit', $row['forum_id'])) ? append_sid($phpbb_root_path . 'posting.' . $phpEx, 'mode=edit&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id']) : ''),
					'U_QUOTE'			=> ($auth->acl_get('f_reply', $row['forum_id'])) ? append_sid($phpbb_root_path . 'posting.' . $phpEx, 'mode=quote&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id']) : '',
					'U_INFO'			=> ($auth->acl_get('m_info', $row['forum_id'])) ? append_sid($phpbb_root_path . 'mcp.' . $phpEx, 'i=main&amp;mode=post_details&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id'], true, $user->session_id) : '',
					'U_DELETE'			=> (!$user->data['is_registered']) ? '' : ((($user->data['user_id'] == $row['user_id'] && $auth->acl_get('f_delete', $row['forum_id']) && $topic_data['topic_last_post_id'] == $row['post_id'] && ($row['post_time'] > time() - ($config['edit_time'] * 60) || !$config['edit_time'])) || $auth->acl_get('m_delete', $row['forum_id'])) ? append_sid($phpbb_root_path . 'posting.' . $phpEx, 'mode=delete&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id']) : ''),

					'U_REPORT'			=> ($auth->acl_get('f_report', $row['forum_id'])) ? append_sid($phpbb_root_path . 'report.' . $phpEx, 'f=' . $row['forum_id'] . '&amp;p=' . $row['post_id']) : '',
					'U_MCP_REPORT'		=> ($auth->acl_get('m_report', $row['forum_id'])) ? append_sid($phpbb_root_path . 'mcp.' . $phpEx, 'i=reports&amp;mode=report_details&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id'], true, $user->session_id) : '',
					'U_MCP_APPROVE'		=> ($auth->acl_get('m_approve', $row['forum_id'])) ? append_sid($phpbb_root_path . 'mcp.' . $phpEx, 'i=queue&amp;mode=approve_details&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id'], true, $user->session_id) : '',
					'U_NOTES'			=> ($auth->acl_getf_global('m_')) ? append_sid($phpbb_root_path . 'mcp.' . $phpEx, 'i=notes&amp;mode=user_notes&amp;u=' . $row['user_id'], true, $user->session_id) : '',
					'U_WARN'			=> ($auth->acl_get('m_warn') && $row['user_id'] != $user->data['user_id'] && $row['user_id'] != ANONYMOUS) ? append_sid($phpbb_root_path . 'mcp.' . $phpEx, 'i=warn&amp;mode=warn_post&amp;f=' . $row['forum_id'] . '&amp;p=' . $row['post_id'], true, $user->session_id) : '',

					'POST_ID'			=> $row['post_id'],
					'POSTER_ID'			=> $row['user_id'],

					'S_HAS_ATTACHMENTS'	=> (!empty($attachments[$row['post_id']])) ? true : false,
					'S_POST_UNAPPROVED'	=> ($row['post_approved']) ? false : true,
					'S_POST_REPORTED'	=> ($row['post_reported'] && $auth->acl_get('m_report', $row['forum_id'])) ? true : false,
					'S_DISPLAY_NOTICE'	=> $row['post_attachment'],// && $display_notice,
				));
			}
		}
	}
}

/**
 * unused viewtopic variable storage
 *
//					'U_MINI_POST'		=> append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'p=' . $row['post_id']) . (($topic_data['topic_type'] == POST_GLOBAL) ? '&amp;f=' . $row['forum_id'] : '') . '#p' . $row['post_id'],
//					'U_NEXT_POST_ID'	=> ($i < $i_total && isset($rowset[$post_list[$i + 1]])) ? $rowset[$post_list[$i + 1]]['post_id'] : '',
//					'U_PREV_POST_ID'	=> $prev_post_id,
//					'S_CUSTOM_FIELDS'	=> (isset($cp_row['row']) && sizeof($cp_row['row'])) ? true : false,
//					'RANK_TITLE'		=> $user_cache[$poster_id]['rank_title'],
//					'RANK_IMG'			=> $user_cache[$poster_id]['rank_image'],
//					'RANK_IMG_SRC'		=> $user_cache[$poster_id]['rank_image_src'],
//					'POSTER_JOINED'		=> $user_cache[$poster_id]['joined'],
//					'POSTER_POSTS'		=> $user_cache[$poster_id]['posts'],
//					'POSTER_FROM'		=> $user_cache[$poster_id]['from'],
//					'POSTER_AVATAR'		=> $user_cache[$poster_id]['avatar'],
//					'POSTER_WARNINGS'	=> $user_cache[$poster_id]['warnings'],
//					'POSTER_AGE'		=> $user_cache[$poster_id]['age'],
//					'SIGNATURE'			=> ($row['enable_sig']) ? $user_cache[$poster_id]['sig'] : '',
//					'EDITED_MESSAGE'	=> $l_edited_by,
//					'BUMPED_MESSAGE'	=> $l_bumped_by,
//					'POST_ICON_IMG'			=> ($topic_data['enable_icons'] && !empty($row['icon_id'])) ? $icons[$row['icon_id']]['img'] : '',
//					'POST_ICON_IMG_WIDTH'	=> ($topic_data['enable_icons'] && !empty($row['icon_id'])) ? $icons[$row['icon_id']]['width'] : '',
//					'POST_ICON_IMG_HEIGHT'	=> ($topic_data['enable_icons'] && !empty($row['icon_id'])) ? $icons[$row['icon_id']]['height'] : '',
//					'ICQ_STATUS_IMG'		=> $user_cache[$poster_id]['icq_status_img'],
//					'ONLINE_IMG'			=> ($poster_id == ANONYMOUS || !$config['load_onlinetrack']) ? '' : (($user_cache[$poster_id]['online']) ? $user->img('icon_user_online', 'ONLINE') : $user->img('icon_user_offline', 'OFFLINE')),
//					'S_ONLINE'				=> ($poster_id == ANONYMOUS || !$config['load_onlinetrack']) ? false : (($user_cache[$poster_id]['online']) ? true : false),
//					'U_PROFILE'		=> $user_cache[$poster_id]['profile'],
//					'U_SEARCH'		=> $user_cache[$poster_id]['search'],
//					'U_PM'			=> ($poster_id != ANONYMOUS && $config['allow_privmsg'] && $auth->acl_get('u_sendpm') && ($user_cache[$poster_id]['allow_pm'] || $auth->acl_gets('a_', 'm_') || $auth->acl_getf_global('m_'))) ? append_sid("{$phpbb_root_path}ucp.$phpEx", 'i=pm&amp;mode=compose&amp;action=quotepost&amp;p=' . $row['post_id']) : '',
//					'U_EMAIL'		=> $user_cache[$poster_id]['email'],
//					'U_WWW'			=> $user_cache[$poster_id]['www'],
//					'U_ICQ'			=> $user_cache[$poster_id]['icq'],
//					'U_AIM'			=> $user_cache[$poster_id]['aim'],
//					'U_MSN'			=> $user_cache[$poster_id]['msn'],
//					'U_YIM'			=> $user_cache[$poster_id]['yim'],
//					'U_JABBER'		=> $user_cache[$poster_id]['jabber'],
//					'S_UNREAD_POST'		=> $post_unread,
//					'S_FIRST_UNREAD'	=> $s_first_unread,
//					'S_TOPIC_POSTER'	=> ($topic_data['topic_poster'] == $row['user_id']) ? true : false,

//					'S_IGNORE_POST'		=> ($row['hide_post']) ? true : false,
//					'L_IGNORE_POST'		=> ($row['hide_post']) ? sprintf($user->lang['POST_BY_FOE'], get_username_string('full', $row['user_id'], $row['username'], $row['user_colour'], $row['post_username']), '<a href="' . $viewtopic_url . "&amp;p={$row['post_id']}&amp;view=show#p{$row['post_id']}" . '">', '</a>') : '',
*/
?>